
/* RCS Information:
 *
 * $Id$
 * $Revision$
 * $Date$
 * $Author$
 * $Locker$
 * 
 */

/******************************************************************************
 *
 *  Copyright (C) 2002, Bill Pierce
 *  This file is part of the hba_scsi_inquiry library.
 *
 *  The hba_scsi_inquiry library is free software; you can 
 *  redistribute it and/or modify it under the terms of the 
 *  GNU Library General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The hba_scsi_inquiry library is distributed in the hope that 
 *  it will be useful, but WITHOUT ANY WARRANTY; without even 
 *  the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 *  PARTICULAR PURPOSE.  See the GNU Library General Public 
 *  License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with the timeout action libary; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Bill Pierce can be contacted at:
 *  7862 NE 148th St.
 *  Kenmore, WA 98028
 *
 * Name -- hba_scsi_inquiry.h 
 *
 * Purpose -- 
 *  Library for performing HBA scsi inquiry functions on top of the 
 *  SNIA Common HBA library.  
 *
 * Remarks --  
 *
 * Known bugs --  
 *
 * Supported Platforms -- Windows NT, 2000, UNIX flavors. 
 *
 * Applicable Invariants -- 
 *
 * Dependencies --
 * Depends on the SNIA Common HBA API version 1.0
 * 
 * Language Version --  Standard C 
 *
 * Author --  Bill Pierce 
 *
 * Origination Date -- 8-21-01
 * 
 *****************************************************************************/

#ifdef WIN32 
 #include <windows.h>
#else
#endif

#include <stdio.h>
#include <string.h>

#include "hba_scsi_inquiry.h"
#include "hbaapiutils.h"       /* Provides some hba api utility functions. */
#include "timeout_action.h"    /* Provides multi-threading support. */
#include "portable_time.h"     /* Provides portable_sleep function. */

/* A structure used to pass arguments and return values between
 * functions executed by the main thread and functions tailored
 * to be executed in separate threads.
 */
typedef struct inquiry_argument {
   HBA_HANDLE              adapter_handle;
   HBA_WWN                 target_port_wwn;
   HBA_UINT64              fcp_lun;
   P_SCSI_INQUIRY_RESPONSE p_inquiry_response;
   int*                    p_return_status;
} INQUIRY_ARG, *P_INQUIRY_ARG;

/* This is a private function used to perform a scsi inquiry in 
 * a spawned thread. It takes a pointer to an INQUIRY_ARG structure
 * and returns a pointer to the result of the synchronous inquiry
 * it performs.
 */
void* do_inquiry_in_thread(void* arg) {
   
   INQUIRY_ARG* p_inquiry_arg;

   p_inquiry_arg = (INQUIRY_ARG*) arg;

   *(p_inquiry_arg->p_return_status) =
   do_synchronous_inquiry(p_inquiry_arg->adapter_handle, 
                          p_inquiry_arg->target_port_wwn, 
                          p_inquiry_arg->fcp_lun, 
                          p_inquiry_arg->p_inquiry_response);

   return (void*) p_inquiry_arg->p_return_status;
}

/* Perform a scsi inquiry.  This method is synchronous, but guaranteed
 * to return within INQUIRY_SYNCHRONOUS_WAIT_LIMIT seconds.
 * Return one of the SCSI_INQUIRY_REPLY values to indicate success or failure. 
 */
int do_synchronous_time_limited_inquiry(
                HBA_HANDLE              adapter_handle,
                HBA_WWN                 target_port_wwn,
                HBA_UINT64              fcp_lun,
                unsigned int            timeout_in_ms,
                P_SCSI_INQUIRY_RESPONSE p_inquiry_response
		                       ) {
   INQUIRY_ARG inquiry_arg;
   int         timeout_action_status;
   int         synchronous_return_status;
   int         return_status;

   /* Initialize return values. */
   return_status = SCSI_INQUIRY_REPLY_ERROR;
   synchronous_return_status = SCSI_INQUIRY_REPLY_ERROR;
   timeout_action_status = TIMEOUT_ACTION_CODE_ERROR;
 
   /* Initialize the argument that will allow the inquiry to be
    * performed in a separate thread.
    */
   inquiry_arg.adapter_handle     = adapter_handle;
   inquiry_arg.target_port_wwn    = target_port_wwn;
   inquiry_arg.fcp_lun            = fcp_lun;
   inquiry_arg.p_inquiry_response = p_inquiry_response;
   inquiry_arg.p_return_status    = &synchronous_return_status;

   /* Perform the inquiry in a timed thread. */ 
   timeout_action_status = timeout_action(do_inquiry_in_thread,
                                           &inquiry_arg,
                                           timeout_in_ms,
                                           (void**) &inquiry_arg.p_return_status
                                          ); 

   /* DEBUG - use this if you want to run single in a single thread.
    * timeout_action_status = TIMEOUT_ACTION_CODE_SUCCESS;
    * inquiry_arg.p_return_status = (int*) do_inquiry_in_thread(&inquiry_arg);
    */
   
   /* Interpret the results and prepare to return. */
   if (TIMEOUT_ACTION_CODE_SUCCESS == timeout_action_status) {
      synchronous_return_status = *(inquiry_arg.p_return_status);
      if ((SCSI_INQUIRY_REPLY_SUCCESS == synchronous_return_status) || 
          (SCSI_INQUIRY_REPLY_EMPTY   == synchronous_return_status)    ) {
	 return_status = synchronous_return_status;
      } 
   } else if (TIMEOUT_ACTION_CODE_TIMEOUT) {
      return_status = SCSI_INQUIRY_REPLY_TIMED_OUT;
   } else {
      fprintf(stderr, "Internal error while performing threaded inquiry.\n");
      return_status = SCSI_INQUIRY_REPLY_ERROR;
   }

   return return_status;

} /* End do_synchronous_time_limited_inquiry. */

/* Perform a scsi inquiry.  This method is synchronous 
 * and will block until the SendScsiInqury() method 
 * in the HBA API returns.  The calling thread to 
 * this method is at the mercy of the library.
 * Return one of the SCSI_INQUIRY_REPLY values to indicate success or failure. 
 */
int do_synchronous_inquiry(
                HBA_HANDLE              adapter_handle,
                HBA_WWN                 target_port_wwn,
                HBA_UINT64              fcp_lun,
                P_SCSI_INQUIRY_RESPONSE p_inquiry_response
		                       ) {

   HBA_STATUS status;
   char       sense_buffer[256]; 
   
   /* Clear the response buffer. */
   memset((char *) p_inquiry_response, 
          '\0', sizeof(SCSI_INQUIRY_RESPONSE)
         );
   
   /* DEBUG - use this if you don't really want to send a ping. 
   status = HBA_STATUS_OK;
   strcpy(p_inquiry_response->VendorId, "Vendor");
   strcpy(p_inquiry_response->ProductId, "Product");
   strcpy(p_inquiry_response->Revision, "rev");
   portable_sleep_ms(500); 
   */

   // Do the inquiry.
   status = HBA_SendScsiInquiry(
                               adapter_handle,
                               target_port_wwn,
                               fcp_lun,
                               0,
                               0,
                               p_inquiry_response,
                               sizeof(*p_inquiry_response),
                               sense_buffer,
                               sizeof(sense_buffer)
                               );
 
   /* Check the return status. */
   if (HBA_STATUS_OK != status) {
      return SCSI_INQUIRY_REPLY_ERROR;
   }

   /* See if we got anything. */
   if (0 == strlen(p_inquiry_response->VendorId)) {
      return SCSI_INQUIRY_REPLY_EMPTY;
   }

   /* If we got here, the call must have succeeded.*/
   return SCSI_INQUIRY_REPLY_SUCCESS;
} /* End do_synchronous_inquiry. */

/* Write that a SCSI Inqiry ping is going out to the target. */
void print_inquiry_going_out(HBA_WWN port_wwn, HBA_UINT64 lun) {
   char port_wwn_string[24];
   int  lun_as_int;
   Lun64ToInt(lun, &lun_as_int);
   WWNToStr(&port_wwn, port_wwn_string);
   printf("\nPinging port %s, LUN %d with SCSI Inquiry:\n\n", 
          port_wwn_string, lun_as_int);
   return;
} /* End print_inquiry_going_out. */

/* Write that a reply was received from the scsi device.*/
void print_ping_reply_received(
		          HBA_WWN                 port_wwn, 
		          P_SCSI_INQUIRY_RESPONSE p_scsi_response,
			  double                  response_time
			      ) {
   char port_wwn_string[24];
   char vendor[9];
   char product[17];

   memset(port_wwn_string, '\0', 24);

   /* The strings in the scsi_response may or may not be null
    * terminated.  This should take care of it.
    */
   memset(vendor, '\0', 9);
   memset(product, '\0', 17);
   strncpy(vendor, p_scsi_response->VendorId, 8);
   strncpy(product, p_scsi_response->ProductId, 16);

   WWNToStr(&port_wwn, port_wwn_string);
   printf("Port %s replies in %2.3f s as %s %s.\n", 
           port_wwn_string,
           response_time,
           vendor,
           product 
         );
   return;
} /* End print_ping_reply_received. */

/* Write that the scsi inquiry request timed out. */
void print_ping_request_timed_out() {
   printf("Request timed out.\n");
   return;
} 

/* Write that the scsi inquiry request came back empty. */
void print_ping_request_reply_empty() {
   printf("Request response was empty.\n");
   return;
} 

/* Write that the scsi inquiry request was unsuccessful. */
void print_ping_request_unsuccessful() {
   printf("Request unsuccessful.\n");
   return;
}

/*****************************************************************************
 *
 *   $Log: $
 *
 *****************************************************************************/
